'define global variables and those to be preserved between calls

'#DEFINE	DEBUGGING	= 1
Private PWM_Mode		as Byte
Private PWM_Pre			as UnsignedInteger
Private PWM_Ready		as Boolean
Private PWM_PhaseA		as Byte
Private PWM_PhaseB		as Byte



Public Function PWMSetFreq(ByVal Freq as Single) as UnsignedInteger
	'the strategy is to determine the mode with the lowest prescaler
	'FPWM Top	= (clock/(pre*freq))-1
	'PCPWM Top	= clock/(1*pre*freq)
	Const TCCR1A_CLR_MSK	as Byte	=	ZX0000_1100
	Const TCCR1B_CLR_MSK	as Byte	=	ZX1110_0000
	
	Const PCPWM_MODE_BITS_A	as Byte	=	ZX0000_0010
	Const PCPWM_MODE_BITS_B	as Byte	=	ZX0001_0000
	
	Const FPWM_MODE_BITS_A	as Byte	=	ZX0000_0010
	Const FPWM_MODE_BITS_B	as Byte	=	ZX0001_1000
	
	Dim Clock		as Single
	Dim Pre_FPWM	as UnsignedInteger
	Dim Pre_PCPWM	as UnsignedInteger	
	Dim PreA		as UnsignedInteger
	Dim PreB		as UnsignedInteger
	
	Dim Old_Count	as UnsignedInteger
	

	PWMSetFreq = 0
	PWM_Mode = 0
	
	If (NOT PWM_Ready) then		'check to see If initialized
		Exit Function
	End If
	

	

	
	Clock = CSng(Register.CPUFrequency)
	Pre_FPWM =  CUInt(  Ceiling(((Clock/Freq)-1.0)/65536.0)  )
	Pre_PCPWM = CUInt(  Ceiling((Clock/(2.0*Freq))/65536.0)  )
	
	#IfDEF DEBUGGING
		Debug.Print "PreFPWM: ";CStr(((Clock/Freq)-1.0)/65536.0)
		Debug.Print "PrePCPWM: ";CStr((Clock/(2.0*Freq))/65536.0)
		Debug.Print "PreFPWM: ";CStr(Pre_FPWM)
		Debug.Print "PrePCPWM: ";CStr(Pre_PCPWM)	
	#ENDIf
	

	PreA = PWMCalcPre(Pre_FPWM)
	PreB = PWMCalcPre(Pre_PCPWM)
	
	#IfDEF DEBUGGING
		Debug.Print "PreA: ";CStr(PreA)
		Debug.Print "PreB: ";CStr(PreB)
	#ENDIf
	
	If (PreA = 0) Then
		If (PreB = 0) Then
			Exit Function		' exit If all options are invalid
		Else
			PWM_Pre = PreB
			PWM_Mode = 2	'Mode 2 must be valid
		End If
	
	ElseIf (PreA = PreB) Then	'neither is invalid, so pick the best
		'With the same prescaler, always choose FPWM
		PWM_Pre = PreA
		PWM_Mode = 1
	Else
		'in case they are different always choose PCPWM because it will have a smaller prescaler
		PWM_Pre = PreB
		PWM_Mode = 2		
	End If





	
	'Clear the necessary bits of TCCR1A and preserve the rest
	Register.TCCR1A = Register.TCCR1A AND TCCR1A_CLR_MSK
	
	'Clear the necessary bits of TCCR1B and preserve the rest, Timer stops counting
	Register.TCCR1B = Register.TCCR1B AND TCCR1B_CLR_MSK
	
	
	If (PWM_Mode = 1) Then
		'use fpwm mode  set Mode before Top
		Register.TCCR1A = Register.TCCR1A OR FPWM_MODE_BITS_A
		Register.TCCR1B = Register.TCCR1B OR FPWM_MODE_BITS_B
		'Set Top
		Register.ICR1 = CUInt(CSng(Register.CPUFrequency)/(Csng(PWM_Pre)*Freq))-1
		If (Register.TCNT1 >= Register.ICR1) Then
			Register.TCNT1 = Register.ICR1-1	'This should minimize glitching
		End If	

		'set the prescaler and timer resumes counting
		Register.TCCR1B = Register.TCCR1B OR PWMGetPreSet(PWM_Pre)	
		
	ElseIf (PWM_Mode = 2) Then
		'use pcpwm mode  set Mode before Top
		Register.TCCR1A = Register.TCCR1A OR PCPWM_MODE_BITS_A
		Register.TCCR1B = Register.TCCR1B OR PCPWM_MODE_BITS_B
		'Set Top
		Register.ICR1 = CUInt(CSng(Register.CPUFrequency)/(2.0*Csng(PWM_Pre)*Freq))
		If (Register.TCNT1 >= Register.ICR1) Then
			Register.TCNT1 = Register.ICR1-1	'This should minimize glitching
		End If
		
		'set the prescaler
		Register.TCCR1B = Register.TCCR1B OR PWMGetPreSet(PWM_Pre)	'timer starts counting now
	End If
	
	PWMSetFreq = PWM_Pre
	
End Function



'____________________________________________________________
Private Function PWMCalcPre(byref Pre as UnsignedInteger) as UnsignedInteger

	PWMCalcPre = 0
	If Pre= 0 Then
		Exit Function
	End If
	
	If (Pre > 1024) Then
		PWMCalcPre = 0				'error MaxFreq too low
	ElseIf (Pre > 256) Then
		PWMCalcPre = 1024
	ElseIf (Pre > 64) Then
		PWMCalcPre = 256
	ElseIf (Pre > 8) Then
		PWMCalcPre = 64
	ElseIf (Pre > 1) Then
		PWMCalcPre = 8
	Else 
		PWMCalcPre = 1
	End If
End Function

'___________________________________________________________________


Private Function PWMGetPreSet(byref Pre as UnsignedInteger) as Byte
	If Pre = 1 Then
		PWMGetPreSet = 1
	ElseIf Pre = 8 Then
		PWMGetPreSet = 2
	ElseIf Pre = 64 Then
		PWMGetPreSet = 3
	ElseIf Pre = 256 Then
		PWMGetPreSet = 4
	ElseIf Pre = 1024 Then
		PWMGetPreSet = 5
	Else
		PWMGetPreSet = 0
	End If
End function




'______________________________________________________________________

Public Function PWMGetMode() as Byte

	PWMGetMode = 0
	
	If (NOT PWM_Ready) then		'check to see If initialized
		Exit Function
	End If
	
	PWMGetMode = PWM_Mode
End Function




'________________________________________________________________________
Public Function PWMGetPre() as UnsignedInteger

	PWMGetPre = 0
	
	If (NOT PWM_Ready) then		'check to see If initialized
		Exit Function
	End If
	
	PWMGetPre = PWM_Pre
End Function



'______________________________________________________________________

Public Function PWMGetFreq()as Single

	PWMGetFreq = 0.0
	
	If (NOT PWM_Ready) then		'check to see If initialized
		Exit Function
	End If
	
	If PWM_Mode = 1 Then
		PWMGetFreq = CSng(Register.CPUFrequency)/(CSng(PWM_Pre)*CSng(Register.ICR1+1))
		
	ElseIf PWM_Mode = 2 Then
		PWMGetFreq = CSng(Register.CPUFrequency)/(2.0*CSng(PWM_Pre)*CSng(Register.ICR1))
	End If
End Function



'______________________________________________________________________

Public Function PWMGetSteps()as UnsignedInteger

	PWMGetSteps = 0
	
	If (NOT PWM_Ready) then		'check to see If initialized
		Exit Function
	End If

	If PWM_Mode > 0 Then
		PWMGetSteps = Register.ICR1
	End If

End Function
	





'______________________________________________________________________
 Public Sub PWMEnable(ByVal Channel as Byte, ByVal Duty as Single, ByVal Phase as Byte)
	Dim Settings as Byte
	
	'channel 1 is OC1A (ZX-24:Pin 26, Green)
	'Channel 2 is OC1B (ZX-24:Pin 27)
	
	If (NOT PWM_Ready) then		'check to see If initialized
		Exit Sub
	End If
	
	If (Phase = 1) then	'Inverted Phase
		If (Channel = 1) Then
			PWM_PhaseA = 1
			Register.OCR1A = CUint(CSng(Register.ICR1) * Duty)
			'Settings = Register.TCCR1A AND ZX0011_1111		'no need to clear as both are being set
			Register.TCCR1A = Register.TCCR1A OR ZX1100_0000		'set the COM1A mode (1:0) only
		ElseIf (Channel = 2) Then
			PWM_PhaseB = 1
			Register.OCR1B = CUint(CSng(Register.ICR1) * Duty)
			'Settings = Register.TCCR1A AND ZX1100_1111		'no need to clear as both are being set
			Register.TCCR1A = Register.TCCR1A OR ZX0011_0000		'set the COM1B mode (1:0) only
		End If
	
	Else	'default to normal phase
		If (Channel = 1) Then
			PWM_PhaseA = 0
			Register.OCR1A = CUint(CSng(Register.ICR1) * Duty)
			Settings = Register.TCCR1A AND ZX0011_1111		'clear the COM1A bits only
			Register.TCCR1A = Settings OR ZX1000_0000		'set the COM1A mode (1:0) only
		ElseIf (Channel = 2) Then
			PWM_PhaseB = 0
			Register.OCR1B = CUint(CSng(Register.ICR1) * Duty)
			Settings = Register.TCCR1A AND ZX1100_1111		'Clear the Com1B bits only
			Register.TCCR1A = Settings OR ZX0010_0000		'set the COM1B mode (1:0) only
		End If
	End If
	
End Sub

'______________________________________________________________________
Public Sub PWMDisable(ByVal Channel as Byte)
	Dim Settings as Byte
	
	If (NOT PWM_Ready) then		'check to see if initialized
		Exit Sub
	End If	
	
	If (Channel = 1) Then
		Register.TCCR1A = Register.TCCR1A AND ZX0011_1111		'Clear the COM1A mode and leave all Else alone
		Call putpin(26,ZXOutputLow)
	ElseIf (Channel = 2) Then
		Register.TCCR1A = Register.TCCR1A AND ZX1100_1111		'Clear the COM1B mode and leave all Else alone
		Call putpin(27,ZXInputTristate)
	End If
	
End Sub


'______________________________________________________________________
Public Sub PWMUnInit ()

	If (NOT PWM_Ready) then		'check to see if initialized
		Exit Sub
	End If
	
	'Turn off the channels
	PWMDisable(1)
	PWMDisable(2)
	
	'don't bother to clear the timer itself
	
	'release the semaphore
	Register.Timer1Busy = False

	
End Sub

'______________________________________________________________________
Public Function PWMInitialize() as Boolean
	'false = failed
	'true = success
	
	'Check to see If Timer1 is busy
	'then reserve Timer1 If it is free, otherwise return with an error
	
	PWMInitialize = False
	PWM_Ready = False
	
	If (NOT Semaphore(Register.Timer1Busy)) then
			'Timer 1 is busy!
		Exit Function
	End If

	Call putpin(26,ZXOutputlow)
	Call putpin(27,ZXOutputLow)
	
	PWMInitialize = True
	PWM_Ready = True
	PWM_Mode = 0
	PWM_Pre = 0
	
End Function

'_______________________________________________
Public Function PWMGetPhase(ByVal Channel as Byte) as Byte
	If (Channel = 2) then			'get channel 2 or B Phase
		PWMGetPhase = PWM_PhaseB
	Else
		PWMGetPhase = PWM_PhaseA	'Default to Channel 1 Phase
	End If

End Function




