Option Explicit

'Use a Potentiometer on ADC_Pin (set to be 14 here) to control the frequency

#DEFINE DEBUGGING	= 1


Public Const Duty_Factor	as Single = 0.178'0.822	
Public Const ADC_Pin		as byte = 14


Public Sub Main()

	Dim Hertz 		as UnsignedInteger
	Dim Old_Hertz	as UnsignedInteger
	Dim FHz			as Single
	
	Dim Counter 	as Integer
	Dim Avg 		as UnsignedLong
	   

	Call PutPin(ADC_Pin,BXInputTristate)
	
	'_______________________________________________________________
	'This is a necessary part of the PWM API
	IF (NOT PWMInitialize()) then
		Debug.print "Timer1 BUSY.  Nothing will work!"
	End If
	
	
	#IFDEF SF_LCD
		Call Delay(2.0)			'give the display time to boot up.
		Call LCDInitialize(LCDPort, 9600, LCDInputPin, LCDOutputPin)
		Call LCDClearScreen()
	#ENDIf



	Old_Hertz = 9999
	DO
		'produce a range of freq from 0.11 to 10,060Hz using a Pot on Pin 14
		'averaging the potentiometer reading will result in less jitter of the reading
		'but still offers adequately quick response to knob movement.

		Avg = 0
		For Counter = 1 to 240
			Avg = Avg + CULng(ShR(GetADC(ADC_Pin),3))
		Next 'Counter
		Hertz = CUInt(Avg\240)	'this is now the ADC count

		If (Hertz <> Old_Hertz) Then
	
			Old_Hertz = Hertz		
			If (Hertz < 10) Then			'0-9
				FHz = CSng(Hertz+1)/10.0	'produce 0.11 - 1
				If (FHz = 0.1) Then
					FHz = 0.11				'minimum freq is 0.11Hz
				End If
				
			ElseIf (Hertz < 19)	Then		'10-18
				FHz = CSng(Hertz-8)			'produce 2-10
				
			Else							'19+
				FHz = CSng((Hertz-17)*10)	'produces 20+ in increments of 10
			End If
			
			

			If PWMSetFreq(FHz) = 0 Then
				'failed to initialize
				#IFDEF DEBUGGING
					Debug.Print "ERROR SETTING PWM"
				#ENDIf
				Exit Sub
			else
				Call PWMEnable(1,Duty_Factor,1)		'Set to Inverted Phase
				Call PWMEnable(2,Duty_Factor,0)		'Set to Normal Phase
			End If
		

		
			#IFDEF DEBUGGING
				Debug.Print "Set Freq: ";Cstr(FHz)
				Debug.Print "Steps: ";cstr(PWMGetSteps)
				Debug.Print "Freq:  ";CStr(PWMGetFreq)
				Debug.Print "Mode:  ";CStr(PWMGetMode)
				Debug.Print "Pre:   ";CStr(PWMGetPre)
				Debug.Print "PhaseA:";CStr(PWMGetPhase(1))
				Debug.Print "PhaseB:";CStr(PWMGetPhase(2))
				Debug.Print
			#ENDIf
		End If
	Loop
	
	'These are commented-out because this program will never reach these lines
	'but they demonstrate the proper usage
	
'	Call PWMDisable(1)	'Turns off this channel
'	Call PWMDisable(2)	'Turns off this channel
'	Call PWMUnInit()	'Necessary to release Timer 1
End Sub